
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: Bagnon, coroutine, IsAddOnLoaded, next, table

local unpack = unpack

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Integration Bagnon")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

M.SKIN_NAME = "SixUI"

M.SPACING = C.BORDER_PADDING * 2

M.items = {}

function M.IterateItems(frameOrSkin)
	return coroutine.wrap(function ()
		for item in next, M.items do
			if not frameOrSkin or item.frame == frameOrSkin or item.frame.profile.skin == frameOrSkin then
				coroutine.yield(item)
			end
		end
	end)
end

function M.IterateOptions()
	return coroutine.wrap(function ()
		if Bagnon and Bagnon.sets then
			for _, options in next, Bagnon.sets.global do
				if type(options) == "table" then
					coroutine.yield(options, "global")
				end
			end

			for realm, owners in next, Bagnon.sets.profiles do
				for id, profile in next, owners do
					for frame, options in next, profile do
						if type(options) == "table" then
							coroutine.yield(options, table.concat({realm, id, frame}, "."))
						end
					end
				end
			end
		end
	end)
end

function M:OnInitialize()
	self.db = D.db:RegisterNamespace(self:GetName(), {global = {options = {}}})
	self:RegisterSkin()
end

function M:OnEnable()
	if not IsAddOnLoaded("Bagnon") then
		self:RegisterEvent("ADDON_LOADED")
		return
	end

	self:RegisterSkin()
	self:ConfigureDefaults()
	self:CreateHooks()
	self:SendMessage("BAGNON_UPDATE_ALL")
end

function M:OnDisable()
	self:ResetItems()
	table.wipe(self.items)
end

function M:ConfigureDefaults()
	for options, optionsID in self.IterateOptions() do
		if not self.db.global.options[optionsID] then
			self.db.global.options[optionsID] = time()
			options.skin = "SixUI"
			options.spacing = 4
		end
	end
end

function M:CreateHooks()
	if Bagnon and Bagnon.Item then
		self:CreateHook("RawHook", Bagnon.Item, "New", "Item_New")
		self:CreateHook("RawHook", Bagnon.Item, "Release", "Item_Release")
	end
end

function M:RegisterSkin()
	if Bagnon and Bagnon.Skins and not Bagnon.Skins:Get(self.SKIN_NAME) then
		Bagnon.Skins:Register({
			id = self.SKIN_NAME,
			template = "BackdropTemplate",
			load = function (...) self:Skin_Load(...) end,
			reset = function (...) self:Skin_Reset(...) end,
			borderColor = function () end,
			centerColor = function () end,
			inset = 0,
			x = 0,
			y = 0,
			x1 = 0,
			y1 = 0,
		})
	end
end

function M:StyleItems(frameOrSkin)
	for item in self.IterateItems(frameOrSkin) do
		self:StyleItem(item)
	end
end

function M:ResetItems(frameOrSkin)
	for item in self.IterateItems(frameOrSkin) do
		self:ResetItem(item)
	end
end

function M:StyleItem(item)
	self:CreateBorder(item, C.BORDER_PADDING):SetBackground(false)
	self:CreateHook("SecureHook", item, "UpdateBorder", "Item_UpdateBorder")
	self:StyleIcon(item.icon)
	self:StyleIcon(item.QuestBang)
	self:StyleIconBorder(item)
end

function M:ResetItem(item)
	self:RemoveBorder(item)
	self:Unhook(item, "UpdateBorder")
	self:ResetIcon(item.icon)
	self:ResetIcon(item.QuestBang)
	self:ResetIconBorder(item)
end

function M:StyleIcon(icon)
	icon:SetTexCoord(unpack(C.ZOOM_TEX_COORD))
end

function M:ResetIcon(icon)
	icon:SetTexCoord(0, 1, 0, 1)
end

function M:StyleIconBorder(item)
	item.IconBorder:Hide()
	item.NormalTexture:Hide()

	if item:GetInfo().iconFileID then
		item:EnableDrawLayer("BORDER")
		item:GetBorder():SetRGB(item.IconBorder:GetVertexColor())
	else
		item:DisableDrawLayer("BORDER")
		item:GetBorder():SetRGB()
	end
end

function M:ResetIconBorder(item)
	item.IconBorder:Show()
	item.NormalTexture:Show()
	item:EnableDrawLayer("BORDER")
	item:UpdateBorder()
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:ADDON_LOADED(event, name)
	if name == "Bagnon" then
		self:UnregisterEvent("ADDON_LOADED")
		self:OnEnable()
	end
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M:Item_New(...)
	local item = self.hooks[Bagnon.Item].New(...)

	self.items[item] = true

	if item.frame.profile.skin == self.SKIN_NAME then
		self:StyleItem(item)
	end

	return item
end

function M:Item_Release(item, ...)
	self.items[item] = nil
	self:ResetItem(item)
	self.hooks[Bagnon.Item].Release(item, ...)
end

function M:Item_UpdateBorder(item)
	self:StyleIconBorder(item)
end

function M:Skin_Load(bg)
	self:CreateBorder(bg)
	self:StyleItems(bg:GetParent())
end

function M:Skin_Reset(bg)
	self:RemoveBorder(bg)
	self:ResetItems(bg:GetParent())
end
